const path = require('path');
const webpack = require('webpack');
const ExtractTextPlugin = require('extract-text-webpack-plugin');

const host = process.env.HOST || '0.0.0.0';
const sourcePath = path.join(__dirname, './src');
const staticPath = path.join(__dirname, '../static');

const OpenBrowserPlugin = require('open-browser-webpack-plugin');

const nodeEnv = process.env.NODE_ENV || 'development';

const isProd = nodeEnv.indexOf('production') !== -1;

const isMock = nodeEnv.indexOf('mock') !== -1;
const port = isMock? 3005 : 3006;

const isProdDebugAllowed = false; // 生产模式亦可调试代码（体积增大几十M）

const extractCSS = new ExtractTextPlugin({
  filename: 'stylesheets/[name].css',
  allChunks: true,
});

const plugins = [
  new webpack.optimize.CommonsChunkPlugin({
    name: ['vendors'],
    minChunks: Infinity,
  }),
  new webpack.DefinePlugin({
    'process.env': {
      NODE_ENV: JSON.stringify(nodeEnv.replace(/(^\s*)|(\s*$)/g, '')),
    },
  }),
  new webpack.ProvidePlugin({
    $: 'jquery',
    jQuery: 'jquery',
    'window.jQuery': 'jquery',
  }),
];

const autoprefixerLoader = {
  loader: 'postcss-loader',
  options: {
    ident: 'postcss',
    plugins: [
      require('autoprefixer')({broswer: 'last 5 versions'}),
    ]
  }
};
const loaders = [
  {
    test: /\.html$/,
    use: [
      {
        loader: 'file-loader?name=../[name].[ext]',
        query: {
          name: '[name].[ext]?[hash]',
        },
      },
    ],
  },
  {
    test: /\.(js|jsx)$/,
    exclude: /node_modules/,
    loader: isProd ? 'babel-loader' : ['babel-loader', 'eslint-loader'],
  },
  {
    test: /\.(jpg|png|gif?)$/,
    loader: isProd ? 'url-loader?limit=8192&name=stylesheets/images/[name].[ext]?[hash]' : 'url-loader?limit=8192',
  },
  {
    test: /\.(ttf|eot|svg|woff|otf?)$/,
    loader: 'file-loader?&name=stylesheets/font/[name].[ext]?[hash]',
  },
  {
    test: /\.ico$/,
    loader: 'file-loader?name=[name].[ext]?[hash]',
  },
  {
    test: /\.css?$/,
    loader: isProd ? extractCSS.extract({
      fallback: 'style-loader',
      use: ['css-loader', autoprefixerLoader],
    }) : ['style-loader', 'css-loader', autoprefixerLoader],
  },
  {
    test: /\.less$/,
    loader: isProd ? extractCSS.extract({
      fallback: 'style-loader',
      use: ['css-loader', autoprefixerLoader, 'less-loader'],
    }) : ['style-loader', 'css-loader', autoprefixerLoader, 'less-loader'],
  },
];

if (isProd) {
  plugins.push(
    new webpack.LoaderOptionsPlugin({
      minimize: true,
      debug: false,
    }),
    new webpack.optimize.UglifyJsPlugin({
      comments: false,
      sourceMap: isProdDebugAllowed,
      compress: {
        warnings: false,
        screw_ie8: true,
        conditionals: true,
        unused: true,
        comparisons: true,
        sequences: true,
        dead_code: true,
        evaluate: true,
        if_return: true,
        join_vars: true,
      },
      output: {
        comments: false,
      },
    }),
    extractCSS
  );
} else {
  plugins.push(
    new webpack.HotModuleReplacementPlugin(),
    new OpenBrowserPlugin({ url: `http://localhost:${port}` })
  );
}

module.exports = {
  devtool: isProd ? (isProdDebugAllowed ? 'source-map' : '') : 'eval',
  entry: {
    bundle: ['babel-polyfill', './src/index'],
    vendors: ['react', 'react-dom', 'react-router', 'react-redux', 'redux', 'redux-thunk', 'jquery'],
  },
  output: {
    path: staticPath,
    filename: 'javascripts/[name].js',
    chunkFilename: 'javascripts/[name].chunk.js',
    publicPath: '/static/',
  },
  plugins,
  node: {
    fs: 'empty',
  },
  module: {
    rules: loaders,
  },
  resolve: {
    extensions: ['.js', '.jsx'],
    modules: [sourcePath, 'node_modules'],
  },
  externals: {
    BMap: 'BMap',
    BMapLib: 'BMapLib',
  },
  devServer: {
    inline: !isProd,
    hot: !isProd,
    contentBase: './src',
    historyApiFallback: true,
    watchContentBase: true,
    host,
    port,
    compress: isProd,
    stats: {
      assets: true,
      children: false,
      chunks: false,
      hash: false,
      modules: false,
      publicPath: false,
      timings: true,
      version: false,
      warnings: true,
      colors: {
        green: '\u001b[32m',
      },
    },
  },
};
